<?php

declare(strict_types=1);

namespace UnifyPayment\Cores;

use JsonException;
use ReflectionException;
use UnifyPayment\Cores\Exceptions\InvalidConfigException;
use UnifyPayment\Cores\Exceptions\RuntimeException;
use UnifyPayment\Cores\Struct\Channels;
use UnifyPayment\Cores\Contracts\Config as ConfigInterface;
use UnifyPayment\Cores\Traits\HttpRequest;

/**
 * Class BasicClient.
 */
class BasicClient
{

    use HttpRequest;

    /**
     * Config
     *
     * @var ConfigInterface|Config
     */
    protected ConfigInterface $config;

    /**
     * Request Domain.
     *
     * @var string
     */
    protected string $domain;

    /**
     * Request payload.
     *
     * @var array
     */
    public array $payload = [];

    /**
     * Payment channels data.
     *
     * @var Channels
     */
    protected Channels $channels;

    /**
     * Channel name
     *
     * @var string
     */
    protected string $channel = 'default';

    /**
     * Application Construct.
     *
     * @param array|Config $config
     *
     * @throws InvalidConfigException
     * @throws ReflectionException
     */
    public function __construct($config)
    {
        $this->config = is_array($config) ? new Config($config) : $config;
        $this->domain = $this->config->get('domain', '');

        $this->setHttpOptions($this->config->get('http', []));
        $this->setChannels();
    }

    /**
     * Get Config
     *
     * @return ConfigInterface
     */
    public function getConfig(): ConfigInterface
    {
        return $this->config;
    }

    /**
     * Set Config.
     *
     * @param ConfigInterface $config
     *
     * @return $this
     */
    public function setConfig(ConfigInterface $config): self
    {
        $this->config = $config;

        return $this;
    }

    /**
     * Get Channels
     *
     * @return Channels
     */
    protected function getChannels(): Channels
    {
        return $this->channels;
    }

    /**
     * Set payment channels.
     *
     * @return void
     *
     * @throws ReflectionException
     * @throws InvalidConfigException
     */
    protected function setChannels(): void
    {
        $channels = $this->config->get('channels');
        if (!array_key_exists($this->channel, $channels)) {
            throw new InvalidConfigException('Missing Unify Payment Config -- [channels]');
        }

        $channels = $channels[$this->channel];

        $this->channels = new Channels();
        $this->channels->setMerchantId($channels['merchantId']);
        $this->channels->setSecret($channels['secret']);
        $this->channels->setOperatorId(reset($channels['operatorId']));
        $this->channels->setChannelId($channels['channelId']);

        $this->payload = array_merge($this->channels->toArray(), $this->payload);
        unset($this->payload['secret']);
    }

    /**
     * Get Operator ID
     *
     * @return string
     */
    public function getChannelsOperatorId(): string
    {
        return $this->getChannels()->getOperatorId();
    }

    /**
     * Set Operator ID
     *
     * @param int $index
     *
     * @return $this
     *
     * @throws InvalidConfigException
     */
    public function setChannelsOperatorId(int $index = 0): self
    {
        $channels = $this->config->get('channels');
        if (!array_key_exists($this->channel, $channels)) {
            throw new InvalidConfigException('Missing Unify Payment Config -- [channels]');
        }

        $channels = $channels[$this->channel];

        if (!array_key_exists($index, $channels['operatorId'])) {
            throw new InvalidConfigException('Missing Unify Payment Config -- [operatorId]-[index:' . $index . ']');
        }

        $this->channels->setOperatorId($channels['operatorId'][$index]);
        $this->payload['operatorId'] = $channels['operatorId'][$index];

        return $this;
    }

    /**
     * Set unify payment channel name.
     *
     * @param string $name
     *
     * @return BasicClient
     *
     * @throws ReflectionException|InvalidConfigException
     */
    public function setChannel(string $name): BasicClient
    {
        $this->channel = $name;
        $this->setChannels();
        return $this;
    }

    /**
     * Get unify payment channel name.
     *
     * @return string
     */
    public function getChannel(): string
    {
        return $this->channel;
    }

    /**
     * Generate Sign.
     *
     * @param array $data
     *
     * @return string
     *
     * @throws InvalidConfigException
     * @throws RuntimeException
     */
    final public function generateSign(array $data): string
    {
        $key = $this->getChannels()->getSecret();

        if (is_null($key)) {
            throw new InvalidConfigException('Missing Unify Payment Config -- [SecretKey]');
        }

        ksort($data);

        return md5($this->getSignContent($data) . $key);
    }

    /**
     * Generate Sign Content.
     *
     * @param array $data
     *
     * @return string
     *
     * @throws RuntimeException
     */
    final protected function getSignContent(array $data): string
    {
        if (array_key_exists('sign', $data)) {
            unset($data['sign']);
        }

        try {
            $buff = json_encode($data, JSON_THROW_ON_ERROR | JSON_UNESCAPED_UNICODE);
        } catch (JsonException $e) {
            throw new RuntimeException('Generate Sign Error:' . $e->getMessage());
        }

        Log::debug('Generate Sign Content', [$data, $buff]);

        return trim($buff);
    }

    /**
     * Set http options
     *
     * @param array $httpOptions
     *
     * @return void
     */
    public function setHttpOptions(array $httpOptions): void
    {
        $this->httpOptions = array_merge($this->httpOptions, $httpOptions, ['base_uri' => $this->domain]);
    }

    /**
     * Send json request.
     *
     * @param string $method
     * @param string $endpoint
     * @param array  $options
     *
     * @return array|string
     */
    public function request(string $method, string $endpoint, array $options = [])
    {
        if ($method === 'post') {
            $options['json'] = $this->payload;
        }

        $requestId = ['requestID' => md5(microtime())];

        Log::debug('Request Content', [$requestId, $method, $endpoint, $options]);

        $response = $this->unwrapResponse($this->getHttpClient()->{$method}($endpoint, $options));

        Log::debug('Response Content', [$requestId, $response]);

        return $response;
    }
}