<?php

declare(strict_types=1);

namespace UnifyPayment\Cores;

use UnifyPayment\Cores\Exceptions\ClassNotExistsException;
use UnifyPayment\Cores\Exceptions\InvalidClientException;

/**
 * Class BasicContainer.
 */
class BasicContainer
{

    /**
     * @var Config
     */
    protected Config $config;

    /**
     * @var array
     */
    protected array $clients = [];

    /**
     * @var array
     */
    protected array $container = [];

    /**
     * Class Constructor.
     *
     * @param array|Config $config
     *
     * @throws ClassNotExistsException
     * @throws InvalidClientException
     */
    public function __construct($config)
    {
        $this->config = is_array($config) ? new Config($config) : $config;
        $this->registerAllClient();
    }

    /**
     * Register All Client.
     *
     * @return void
     *
     * @throws InvalidClientException
     * @throws ClassNotExistsException
     */
    private function registerAllClient(): void
    {
        foreach ($this->clients as $key => $value) {
            if (!class_exists($value)) {
                throw new ClassNotExistsException('Class [' . $value . '] Not Found.', $value);
            }

            if (!is_subclass_of($value, BasicClient::class)) {
                throw new InvalidClientException('Invalid Client [' . $value . '].', $value);
            }

            $this->container[$key] = new $value($this->config);
        }
    }

    /**
     * Register Other Client.
     *
     * @param string $name
     * @param string $client
     *
     * @return void
     *
     * @throws ClassNotExistsException
     * @throws InvalidClientException
     */
    public function registerOtherClient(string $name, string $client): void
    {
        if (!class_exists($client)) {
            throw new ClassNotExistsException('Class [' . $client . '] Not Found.', $client);
        }

        if (!is_subclass_of($client, BasicClient::class)) {
            throw new InvalidClientException('Invalid Client [' . $client . '].', $client);
        }

        $this->container[$name] = new $client($this->config);
    }

    /**
     * Get Magic Property.
     *
     * @param string $name
     *
     * @return BasicClient
     *
     * @throws InvalidClientException
     */
    public function __get(string $name): BasicClient
    {
        if (!isset($this->container[$name])) {
            throw new InvalidClientException($name);
        }

        return $this->container[$name];
    }

    /**
     * Set Magic Property.
     *
     * @param string      $name
     * @param BasicClient $value
     *
     * @return void
     *
     * @throws InvalidClientException
     */
    public function __set(string $name, BasicClient $value)
    {
        if (isset($this->container[$name])) {
            throw new InvalidClientException('Override Client [' . $name . '] Not Allowed.', [$name, $value]);
        }

        $this->container[$name] = $value;
    }

    /**
     * Isset Magic Property.
     *
     * @param string $name
     *
     * @return bool
     */
    public function __isset(string $name)
    {
        return (bool)$this->container[$name];
    }
}
